#!/bin/bash
###########################################################################
#                                                                         #
#                                                                         #
#              Copyright (c) 2008 IskraTEL                                #
#                                                                         #
#                                                                         #
#                                                                         #
#   Name        : snmpcomm.sh                                             #
#                                                                         #
#   Description : Script for managing SNMP community names                #
#                                                                         #
#   Date        : Oct 2012                                                #
#                                                                         #
#   Author      : Janez Robic                                             #
#                                                                         #
#   Translation : * None                                                  #
#                                                                         #
#   Remarks     : * None                                                  #
#                                                                         #
#   Revisions   : * None                                                  #
#                                                                         #
###########################################################################

#commands
LOGGER="/usr/bin/logger -p user.alert -t traps.sh --"
isSystemd=`/sbin/cfg-isSystemdInit.sh`
SNMPD_DAEMON=/etc/init.d/snmpd


#config files
SNMP_FILE=/etc/snmp/snmpd.conf
SNMP_FILE_1=/etc/snmp/snmpd.conf.1
SNMP_VAR_FILE=/var/lib/net-snmp/snmpd.conf

write_snmp_file ()
{
   if [ -s $SNMP_FILE_1 ]
   then
      mv $SNMP_FILE_1 $SNMP_FILE
   else
      echo "Unable to write $SNMP_FILE !"  | $LOGGER
   fi
}

update_communities()
{
   [ -z $COMM ] && usage "Empty community name not allowed!"
   COMMUNITY_IN_FILE="`grep "${ACCESS}community.*$COMM$" $SNMP_FILE`"
   case $ACTION in
      insert)
         if [ "$COMMUNITY_IN_FILE" != "" ]; then
            echo "$ACCESS community $COMM already exist!" >&2
            exit 1
         else
            #delete all old communities 
            sed "/^${ACCESS}community */d" $SNMP_FILE > $SNMP_FILE_1
            write_snmp_file
            echo "${ACCESS}community $COMM" >> $SNMP_FILE
         fi;;
      delete)
         if [ "$COMMUNITY_IN_FILE" = "" ]; then 
            echo "$ACCESS community $COMM not found!" >&2
            exit 1
         else 
            sed "/${ACCESS}community *$COMM$/d" $SNMP_FILE > $SNMP_FILE_1
            write_snmp_file
         fi;;
      *)
         echo "Wrong action: $ACTION"
         exit 2;;
   esac
   snmpd_daemon reload 
}

snmpd_daemon()
{
   if [ "$isSystemd" == "NO" ]; then 
      $SNMPD_DAEMON $1
   else
      systemctl $1 snmpd.service
   fi
}

usage()
{
   prog=`basename $0`
   [ "$1" != "" ] && echo $1 >&2 && echo >&2
   echo "Usage: $prog --insert|-i rw|ro <community_name>" >&2
   echo "       $prog --delete|-d rw|ro <community_name>" >&2
   echo "       $prog --show|-s" >&2
   echo "       $prog --help|-h" >&2
   echo "v3 usage: $prog --v3|-3 --insert|-i <user> [-a MD5|SHA authPasswd [-p DES|AES privPasswd]] [-m ro|rw]" >&2
   echo "          $prog --v3|-3 --delete|-d <user>" >&2
   echo "          $prog --v3|-3 --show|-s" >&2
   exit 1
}

show_communities()
{
   echo "ro_community:`grep rocommunity $SNMP_FILE|grep -v \^#|awk '{print $2}'`"
   echo "rw_community:`grep rwcommunity $SNMP_FILE|grep -v \^#|awk '{print $2}'`"
}

show_v3users()
{
   SNMP_USERS=$(\
   awk '/^usmUser/ {
      if ($8 == ".1.3.6.1.6.3.10.1.1.1")
         AUTH="none"
      else if ($8 == ".1.3.6.1.6.3.10.1.1.2")
         AUTH="MD5"
      else if ($8 = ".1.3.6.1.6.3.10.1.1.3")
         AUTH="SHA"
      if ($10 == ".1.3.6.1.6.3.10.1.2.1")
         PRIV="none"
      else if ($10 == ".1.3.6.1.6.3.10.1.2.2")
         PRIV="DES"
      else if ($10 == ".1.3.6.1.6.3.10.1.2.4")
         PRIV="AES"
      print $5 " " AUTH " " PRIV 
   }' $SNMP_VAR_FILE 2>/dev/null | tr -d '"') 
   while read line; do 
      U=$(echo $line|awk '{print $1}')
      ACCESS=$(grep -w $U $SNMP_FILE|sed "s/^rouser.*/ro/; s/^rwuser.*/rw/")
      echo "$line $ACCESS"
   done <<< "$SNMP_USERS" 
}

update_v3users()
{
   #check if user exist in /etc/snmp/snmpd.conf
   UEXIST=$(grep -w "^r[ow]user[ \t]*$V3USER" $SNMP_FILE)
   #and in /var/lib/net-snmp/snmpd.conf
   UEXIST="$UEXIST$(grep "^usmUser\|^createUser" $SNMP_VAR_FILE|grep -w $V3USER)"
   if [ "$ACTION" = "insert" ]; then 
      [ "$AUTH_PROT" = "" -a "$PRIV_PROT" != "" ] && usage "user must use authentication to be able to use privacy" 
      if [ "$UEXIST" != "" ]; then 
         echo "User $V3USER already exist!" >&2
         exit 1
      fi
      [ "$AUTH_PROT" != "" ] && AUTH="$AUTH_PROT \"$AUTH_PASSWD\""
      [ "$PRIV_PROT" != "" ] && PRIV="$PRIV_PROT \"$PRIV_PASSWD\""
      [ "$AUTH_PROT" = "" -a "$PRIV_PROT" = "" ] && ETC_SNMPD_AUTH="noauth"
      snmpd_daemon stop
      echo "$ACCESS_MODE $V3USER $ETC_SNMPD_AUTH" >> $SNMP_FILE
      echo "createUser \"$V3USER\" $AUTH $PRIV" >> $SNMP_VAR_FILE
      sleep 1
      snmpd_daemon start
   fi
   if [ "$ACTION" = "delete" ]; then 
      if [ "$UEXIST" = "" ]; then 
         echo "User $V3USER not found!" >&2
         exit 1
      fi      
      snmpd_daemon stop
      sleep 1
      grep -vw "^r[ow]user[ \t]*${V3USER}" $SNMP_FILE > $SNMP_FILE_1
      write_snmp_file
      sed -i "/^createUser.*\"${V3USER}\"/d; /^usmUser.*\"${V3USER}\"/d" $SNMP_VAR_FILE
      sleep 1
      snmpd_daemon start
   fi
   exit 0
}

parseV3params()
{
   case $1 in 
      --insert|-i) ACTION="insert";;
      --delete|-d) ACTION="delete";;
      --show|-s) 
         show_v3users
         exit 0 ;;
      *) usage "Wrong action ($1)";;
   esac

   V3USER=$2
   shift 2

   if [ "$ACTION" = "delete" -a $# -ne 0 ]; then 
      NUM=$#
      ((NUM+=3))
      usage "Wrong nuber of arguments ($NUM)" 
   fi


   AUTH_PROT=""; AUTH_PASSWD=""
   PRIV_PROT=""; PRIV_PASSWD=""
   ACCESS_MODE="rwuser"
   while [ $# -gt 0 ]; do 
      case $1 in 
         -a)  #auth protocol+password
            shift
            case $1 in 
               MD5|md5) AUTH_PROT=MD5;; 
               SHA|sha) AUTH_PROT=SHA;; 
               none)                 ;; #valid
               *) usage "Wrong authentication protocol ($1)";;
            esac
            if [ "$AUTH_PROT" != "" ]; then 
               [ "$(echo $2|wc -c)" -le 8 ] && usage "Authentication password too short (must be at least 8 chars)"
               AUTH_PASSWD=$2
            fi
            shift 2
            ;;
          -p) #privacy protocol+password  
            shift
            case $1 in 
               DES|des) PRIV_PROT=DES;; 
               AES|aes) PRIV_PROT=AES;; 
               none)                 ;; #valid
               *) usage "Wrong privacy protocol ($1)";;
            esac
            if [ "$PRIV_PROT" != "" ]; then 
               [ "$(echo $2|wc -c)" -le 8 ] && usage "Privacy password too short (must be at least 8 chars)"
               PRIV_PASSWD=$2
            fi
            shift 2
            ;;
         -m) # access mode (ro|rw)
            shift
            case $1 in 
               ro|RO) ACCESS_MODE=rouser;;
               rw|RW) ACCESS_MODE=rwuser;;
               *) usage "Wrong access mode ($1)";;
            esac
            shift 
            ;;
         *) usage "Wrong arg ($1)";;
      esac
   done 
}

#main script
ACTION=""; COMM=""; ACCESS=""
V3=""

#parse input parameters
if [ "$1" = "--v3" -o "$1" = "-3" ]; then 
   case $# in 
      2|3|5|6|8|9|11) ;; #pass
      *) usage "Wrong number of parameters ($#)";;
   esac
else
[ $# -ne 1 -a $# -ne 3 ] && usage "Wrong number of parameters ($#)"
fi
case $1 in
   --insert|-i) 
      ACTION="insert"
      ACCESS=$2
      COMM=$3;;
   --delete|-d) 
      ACTION="delete"
      ACCESS=$2
      COMM=$3;;
   --v3|-3) 
      V3=v3
      shift
      parseV3params $@
      update_v3users;;
   --show|-s) 
      show_communities
      exit 0;;
   --help|-h) 
      usage;;
   *) usage "Wrong action - $1!";;
esac
update_communities

exit 0

